from setuptools import setup, find_packages


setup(
    name='star-lighting',
    version='0.2',
    author='duxinn',
    author_email="xiamin_1990@yahoo.com",
    description='simplely use python',
    long_description='simplely use python',
    packages=['StarLighting'],
    install_requires=[],
)





